"""
Configs migration

Generation datetime: 2023-08-02 09:01:15.394089
"""

import json
from typing import List

from configs_core.migration.db_migrate_helper import (
    DatabaseMigrationHelper,
    ExternalLimitationToModify,
    LimitationToModify,
)
from configs_core.migration.models import Setting
from sqlalchemy import and_, select

mh: DatabaseMigrationHelper

REVISION = "6254472d"
DOWN_REVISION = "d6a13b60"


limitationsToModify: List[LimitationToModify] = []

externalLimitationsToModify: List[ExternalLimitationToModify] = []


def getDefaultSetting() -> dict:
    """
    Replace default value 'TASKS_REDIS_DB_ADDRESS' by 'LUNA_ATTRIBUTES_DB' setting value
    """
    filters = and_(Setting.name == "LUNA_ATTRIBUTES_DB", Setting.is_default == True)
    selectStatement = select([Setting.value]).where(filters)
    selectResults = mh.connection.execute(selectStatement).cursor.fetchone()
    return json.loads(selectResults[0])


def upgrade():
    """Upgrade configs to new revision"""

    # limitations to remove
    mh.removeLimitation(limitationName="LUNA_TASKS_WORKER_ADDRESS")
    mh.removeLimitation(limitationName="LUNA_TASKS_WORKER_TIMEOUTS")
    mh.removeLimitation(limitationName="TASKS_ROUTER_TO_WORKER_REQUESTS_CONCURRENCY")
    mh.removeLimitation(limitationName="WORKERS_COUNT_PER_TASK_TYPE")

    # limitations to add
    mh.addLimitation(
        limitationName="TASKS_REDIS_DB_ADDRESS",
        validationSchema={
            "type": "object",
            "properties": {
                "host": {"type": "string"},
                "port": {"type": "integer", "minimum": 1, "maximum": 65535},
                "password": {"type": "string"},
                "sentinel": {
                    "description": "Redis sentinels settings",
                    "type": "object",
                    "properties": {
                        "master_name": {"type": "string"},
                        "sentinels": {
                            "type": "array",
                            "items": {
                                "description": "Redis sentinel settings. Host+port",
                                "type": "string",
                                "pattern": "[a-zA-Z0-9.]+:\\d{1,5}",
                                "example": "127.0.0.1:26379",
                            },
                        },
                    },
                    "additionalProperties": False,
                    "required": ["master_name", "sentinels"],
                },
                "number": {
                    "type": "integer",
                    "enum": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14],
                },
            },
            "additionalProperties": False,
        },
        defaultValue=getDefaultSetting(),
        description="Redis database address",
        services=["luna-tasks"],
        groupNames=["luna-platform"],
    )

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)


def downgrade():
    """Downgrade configs to previous revision"""

    # limitations to modify
    for limitation in externalLimitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to modify
    for limitation in limitationsToModify:
        mh.migrateLimitation(limitation)

    # limitations to add
    mh.addLimitation(
        limitationName="LUNA_TASKS_WORKER_ADDRESS",
        validationSchema={
            "type": "string",
            "pattern": "^http(s)?:\\/\\/[^\\/:]+(:\\d+)?$",
            "example": "http://127.0.0.1:5051",
        },
        defaultValue="http://127.0.0.1:5051",
        description="Luna Tasks Worker address",
        services=["luna-tasks"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="LUNA_TASKS_WORKER_TIMEOUTS",
        validationSchema={
            "type": "object",
            "properties": {
                "request": {"type": "number", "exclusiveMinimum": 0},
                "connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_connect": {"type": "number", "exclusiveMinimum": 0},
                "sock_read": {"type": "number", "exclusiveMinimum": 0},
            },
            "description": "timeouts in seconds",
            "additionalProperties": False,
        },
        defaultValue={
            "request": 600,
            "connect": 20,
            "sock_connect": 10,
            "sock_read": 600,
        },
        description="timeouts for http-request to luna-tasks-worker service",
        services=["luna-tasks"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="TASKS_ROUTER_TO_WORKER_REQUESTS_CONCURRENCY",
        validationSchema={"type": "integer"},
        defaultValue=10,
        description="Concurrency for sending subtasks to luna-tasks-workers",
        services=["luna-tasks"],
        groupNames=["luna-platform"],
    )
    mh.addLimitation(
        limitationName="WORKERS_COUNT_PER_TASK_TYPE",
        validationSchema={"type": "integer"},
        defaultValue=5,
        description="Workers' concurrency: number of workers for every task type",
        services=["luna-tasks"],
        groupNames=["luna-platform"],
    )

    # limitations to remove
    mh.removeLimitation(limitationName="TASKS_REDIS_DB_ADDRESS")
